@@ -10,11 +10,11 @@ from logilab.common.decorators import monkeypatch
 from logilab.mtconverter import xml_escape
 
 from cubicweb import Unauthorized
-from cubicweb.selectors import implements, rql_condition, relation_possible
+from cubicweb.predicates import is_instance, rql_condition, relation_possible
 from cubicweb.view import EntityView
-from cubicweb.mixins import TreePathMixIn
 from cubicweb.web import stdmsgs, uicfg, component, facet
-from cubicweb.web.views import primary, basecontrollers
+from cubicweb.web.views import primary, basecontrollers, treeview
+from cubicweb.web.views.ajaxcontroller import ajaxfunc
 
 
 _pvs = uicfg.primaryview_section
@@ -32,7 +32,7 @@ _abaa.tag_object_of(('Keyword', 'subkeyword_of', 'Keyword'), True)
 # classification views ########################################################
 
 class ClassificationPrimaryView(primary.PrimaryView):
-    __select__ = implements('Classification')
+    __select__ = is_instance('Classification')
 
     def render_entity_attributes(self, entity):
         pass
@@ -47,7 +47,7 @@ class ClassificationPrimaryView(primary.PrimaryView):
 # keyword views ###############################################################
 
 class KeywordPrimaryView(primary.PrimaryView):
-    __select__ = implements('Keyword')
+    __select__ = is_instance('Keyword')
 
     def cell_call(self, row, col, **kwargs):
         entity = self.cw_rset.get_entity(row, col)
@@ -57,10 +57,10 @@ class KeywordPrimaryView(primary.PrimaryView):
         self.wview('treeview', rset, 'null')
 
 
-class KeywordComboBoxView(TreePathMixIn, EntityView):
+class KeywordComboBoxView(treeview.TreePathView):
     """display keyword in edition's combobox"""
     __regid__ = 'combobox'
-    __select__ = implements('Keyword', 'Classification')
+    __select__ = is_instance('Keyword', 'Classification')
 
     item_vid = 'text'
     separator = u' > '
@@ -83,7 +83,7 @@ class SkosView(EntityView):
     __regid__ = 'skos'
     content_type = 'application/xml'
     templatable = False
-    __select__ = implements('Keyword', 'Classification')
+    __select__ = is_instance('Keyword', 'Classification')
 
     def call(self, **kwargs):
         self.w(SKOS_OPENING_ROOT)
@@ -97,7 +97,7 @@ class SkosView(EntityView):
 class SkosItemView(EntityView):
     __regid__ = 'skositemview'
     content_type = 'application/xml'
-    __select__ = implements('Keyword', 'Classification')
+    __select__ = is_instance('Keyword', 'Classification')
 
     def cell_call(self, row, col, show_parent=True, stop=False):
         w = self.w
@@ -131,7 +131,7 @@ class KeywordBarVComponent(component.EntityVComponent):
 
     def get_keywords(self):
         """helper method for subclasses redefinition"""
-        entity = self.entity(0)
+        entity = self.cw_rset.get_entity(0, 0)
         return entity.related('applied_to', 'object')
 
     def call(self, **kwargs):
@@ -157,10 +157,9 @@ class AddKeywordVComponent(component.EntityVComponent):
     order = 153
     htmlclass = 'navigation'
 
-    def call(self):
+    def entity_call(self, entity, view=None):
         self.add_js(['cubicweb.widgets.js', 'cubes.keyword.js'])
         self._cw.add_css('cubicweb.suggest.css')
-        entity = self.entity(0)
         self.w(u'<table><tr><td>')
         self.w(u'<a class="button sglink" href="javascript: showKeywordSelector(%s, \'%s\', \'%s\');">%s</a></td>' % (
             entity.eid, self._cw._(stdmsgs.BUTTON_OK[0]),
@@ -182,9 +181,11 @@ class AppliedToFacet(facet.RelationFacet):
         vocab = []
         scheme = None
         for e in sorted(rset.entities(),
-                        key=lambda e: (e.classification.name, e.view('combobox'))):
-            if scheme != e.classification.name:
-                vocab.append( (_(e.classification.name), None) )
+                        key=lambda e: (e.cw_adapt_to('ITree').classification.name,
+                                       e.view('combobox'))):
+            classification_name = e.cw_adapt_to('ITree').classification.name
+            if scheme != classification_name:
+                vocab.append( (_(classification_name), None) )
             vocab.append( (e.view('combobox'), e.eid) )
         return vocab
 
@@ -242,16 +243,14 @@ class ClassificationFacet(facet.RelationFacet):
 
 # add some classification schema related methods to the Jsoncontroller ########
 
-@monkeypatch(basecontrollers.JSonController)
-@basecontrollers.jsonize
+@ajaxfunc(output_type='json')
 def js_possible_keywords(self, eid):
     rql = ('DISTINCT Any N WHERE K is Keyword, K name N, NOT K applied_to X, '
            'X eid %(x)s, K included_in C, C classifies ET, X is ET')
     rset = self.cursor.execute(rql, {'x' : eid, 'u' : self._cw.user.eid}, 'x')
     return [name for (name,) in rset]
 
-@monkeypatch(basecontrollers.JSonController)
-@basecontrollers.jsonize
+@ajaxfunc(output_type='json')
 def js_add_keywords(self, eid, kwlist):
     msg = self._cw._('keywords applied')
     kwrset = self.cursor.execute('Any K,N,C WHERE K is Keyword, K name N, K included_in C, '
